# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import pydantic_v1
from ..core.query_encoder import encode_query
from ..core.remove_none_from_dict import remove_none_from_dict
from ..core.request_options import RequestOptions
from ..generated_types.types.country import Country
from ..generated_types.types.risk import Risk
from ..shared_errors.errors.bad_request import BadRequest
from ..shared_errors.errors.internal_server_error import InternalServerError
from ..shared_errors.errors.method_not_allowed import MethodNotAllowed
from ..shared_errors.errors.not_found import NotFound
from ..shared_errors.errors.rate_limit_exceeded import RateLimitExceeded
from ..shared_errors.errors.unauthorized import Unauthorized
from ..shared_errors.types.bad_request_response import BadRequestResponse
from ..shared_errors.types.internal_server_error_response import InternalServerErrorResponse
from ..shared_errors.types.method_not_allowed_response import MethodNotAllowedResponse
from ..shared_errors.types.not_found_response import NotFoundResponse
from ..shared_errors.types.rate_limit_response import RateLimitResponse
from ..shared_errors.types.unauthorized_response import UnauthorizedResponse
from .types.upstream_trade_traversal_response import UpstreamTradeTraversalResponse


class SupplyChainClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def upstream_trade_traversal(
        self,
        id: str,
        *,
        countries: typing.Optional[typing.Sequence[Country]] = None,
        not_countries: typing.Optional[typing.Sequence[Country]] = None,
        risks: typing.Optional[typing.Sequence[Risk]] = None,
        not_risk: typing.Optional[typing.Sequence[Risk]] = None,
        hs_code: typing.Optional[typing.Sequence[str]] = None,
        not_hs_code: typing.Optional[typing.Sequence[str]] = None,
        components: typing.Optional[typing.Sequence[str]] = None,
        not_components: typing.Optional[typing.Sequence[str]] = None,
        max_depth: typing.Optional[int] = None,
        date: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpstreamTradeTraversalResponse:
        """
        <Warning>This endpoint is in beta and is subject to change. It is provided for early access and testing purposes only.</Warning> Execute a traversal of the upstream trade network (supply chain) of an entity, returning a set of entities and edges between them.

        Parameters
        ----------
        id : str
            The root entity identifier.

        countries : typing.Optional[typing.Sequence[Country]]
            Country leaf node filter. Only return supply chains that end with a supplier in 1+ of the specified countries.

        not_countries : typing.Optional[typing.Sequence[Country]]
            Country leaf node filter. Only return supply chains that end with a supplier in none of the specified countries.

        risks : typing.Optional[typing.Sequence[Risk]]
            Risk leaf node filter. Only return supply chains that end with a supplier that has 1+ of the specified risk factors.

        not_risk : typing.Optional[typing.Sequence[Risk]]
            Risk leaf node filter. Only return supply chains that end with a supplier that has none of the specified risk factors.

        hs_code : typing.Optional[typing.Sequence[str]]
            Product root edge filter. Only return supply chains that start with an edge that has 1+ of the specified HS codes.

        not_hs_code : typing.Optional[typing.Sequence[str]]
            Product root edge filter. Only return supply chains that start with an edge that has none of the specified HS codes.

        components : typing.Optional[typing.Sequence[str]]
            Component node filter. Only return supply chains that contain at least one edge with 1+ of the specified HS codes.

        not_components : typing.Optional[typing.Sequence[str]]
            Component node filter. Only return supply chains that contain no edges with any of the specified HS codes.

        max_depth : typing.Optional[int]
            The maximum depth of the traversal, from 1 to 4 inclusive. Default is 4. Reduce if query is timing out.

        date : typing.Optional[str]
            The date range to filter the supply chain by by only considering shipments within the specified date range, inclusive. The date range is formatted as "YYYY-MM-DD|YYYY-MM-DD", where the first date is the start date and the second date is the end date. Both dates are optional, e.g. "|2022-01-01" will return all shipments up to and including 2022-01-01.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpstreamTradeTraversalResponse

        Examples
        --------
        from sayari.client import Sayari

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.supply_chain.upstream_trade_traversal(
            id="ESkH7J-UCRfY5t0_JXIH3w",
            date="2023-06-01",
            hs_code=["3206"],
            components=["3204"],
            max_depth=2,
            risks=["forced_labor_uflpa_origin_subtier"],
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            method="GET",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"v1/upstream/{jsonable_encoder(id)}"),
            params=encode_query(
                jsonable_encoder(
                    remove_none_from_dict(
                        {
                            "countries": jsonable_encoder(countries),
                            "-countries": jsonable_encoder(not_countries),
                            "risks": jsonable_encoder(risks),
                            "-risks": jsonable_encoder(not_risk),
                            "hs_code": jsonable_encoder(hs_code),
                            "-hs_code": jsonable_encoder(not_hs_code),
                            "components": jsonable_encoder(components),
                            "-components": jsonable_encoder(not_components),
                            "max_depth": max_depth,
                            "date": date,
                            **(
                                request_options.get("additional_query_parameters", {})
                                if request_options is not None
                                else {}
                            ),
                        }
                    )
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(UpstreamTradeTraversalResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFound(pydantic_v1.parse_obj_as(NotFoundResponse, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowed(pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncSupplyChainClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def upstream_trade_traversal(
        self,
        id: str,
        *,
        countries: typing.Optional[typing.Sequence[Country]] = None,
        not_countries: typing.Optional[typing.Sequence[Country]] = None,
        risks: typing.Optional[typing.Sequence[Risk]] = None,
        not_risk: typing.Optional[typing.Sequence[Risk]] = None,
        hs_code: typing.Optional[typing.Sequence[str]] = None,
        not_hs_code: typing.Optional[typing.Sequence[str]] = None,
        components: typing.Optional[typing.Sequence[str]] = None,
        not_components: typing.Optional[typing.Sequence[str]] = None,
        max_depth: typing.Optional[int] = None,
        date: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpstreamTradeTraversalResponse:
        """
        <Warning>This endpoint is in beta and is subject to change. It is provided for early access and testing purposes only.</Warning> Execute a traversal of the upstream trade network (supply chain) of an entity, returning a set of entities and edges between them.

        Parameters
        ----------
        id : str
            The root entity identifier.

        countries : typing.Optional[typing.Sequence[Country]]
            Country leaf node filter. Only return supply chains that end with a supplier in 1+ of the specified countries.

        not_countries : typing.Optional[typing.Sequence[Country]]
            Country leaf node filter. Only return supply chains that end with a supplier in none of the specified countries.

        risks : typing.Optional[typing.Sequence[Risk]]
            Risk leaf node filter. Only return supply chains that end with a supplier that has 1+ of the specified risk factors.

        not_risk : typing.Optional[typing.Sequence[Risk]]
            Risk leaf node filter. Only return supply chains that end with a supplier that has none of the specified risk factors.

        hs_code : typing.Optional[typing.Sequence[str]]
            Product root edge filter. Only return supply chains that start with an edge that has 1+ of the specified HS codes.

        not_hs_code : typing.Optional[typing.Sequence[str]]
            Product root edge filter. Only return supply chains that start with an edge that has none of the specified HS codes.

        components : typing.Optional[typing.Sequence[str]]
            Component node filter. Only return supply chains that contain at least one edge with 1+ of the specified HS codes.

        not_components : typing.Optional[typing.Sequence[str]]
            Component node filter. Only return supply chains that contain no edges with any of the specified HS codes.

        max_depth : typing.Optional[int]
            The maximum depth of the traversal, from 1 to 4 inclusive. Default is 4. Reduce if query is timing out.

        date : typing.Optional[str]
            The date range to filter the supply chain by by only considering shipments within the specified date range, inclusive. The date range is formatted as "YYYY-MM-DD|YYYY-MM-DD", where the first date is the start date and the second date is the end date. Both dates are optional, e.g. "|2022-01-01" will return all shipments up to and including 2022-01-01.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpstreamTradeTraversalResponse

        Examples
        --------
        from sayari.client import AsyncSayari

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        await client.supply_chain.upstream_trade_traversal(
            id="ESkH7J-UCRfY5t0_JXIH3w",
            date="2023-06-01",
            hs_code=["3206"],
            components=["3204"],
            max_depth=2,
            risks=["forced_labor_uflpa_origin_subtier"],
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            method="GET",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"v1/upstream/{jsonable_encoder(id)}"),
            params=encode_query(
                jsonable_encoder(
                    remove_none_from_dict(
                        {
                            "countries": jsonable_encoder(countries),
                            "-countries": jsonable_encoder(not_countries),
                            "risks": jsonable_encoder(risks),
                            "-risks": jsonable_encoder(not_risk),
                            "hs_code": jsonable_encoder(hs_code),
                            "-hs_code": jsonable_encoder(not_hs_code),
                            "components": jsonable_encoder(components),
                            "-components": jsonable_encoder(not_components),
                            "max_depth": max_depth,
                            "date": date,
                            **(
                                request_options.get("additional_query_parameters", {})
                                if request_options is not None
                                else {}
                            ),
                        }
                    )
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(UpstreamTradeTraversalResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFound(pydantic_v1.parse_obj_as(NotFoundResponse, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowed(pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



import typing
import httpx
import urllib.parse
import csv
import queue
import time
from multiprocessing import Process, Queue, set_start_method
from .base_client import BaseClient, AsyncBaseClient
from .environment import SayariEnvironment

# resolution attributes
Name = "name"
Identifier = "identifier"
Country = "country"
Address = "address"
DOB = "dateofbirth"
Contact = "contact"
Type = "type"

# CSV Headers
csvDict = {
    Name: "The name of the entity",
    Identifier: "...",
    Country: "Must be from the enum set",
    Address: "...",
    DOB: "...",
    Contact: "...",
    Type: "Must be from the enum set",
}


class Sayari(BaseClient):
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propogate to these functions.

    Parameters:
        - base_url: typing.Optional[str]. The base url to use for requests from the client.

        - environment: ClientEnvironment. The environment to use for requests from the client. from .environment import ClientEnvironment

                                          Defaults to ClientEnvironment.PRODUCTION

        - client_name: str.

        - timeout: typing.Optional[float]. The timeout to be used, in seconds, for requests by default the timeout is 60 seconds, unless a custom httpx client is used, in which case a default is not set.

        - follow_redirects: typing.Optional[bool]. Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

        - httpx_client: typing.Optional[httpx.Client]. The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

        - client_id: str.

        - client_secret: str.
    ---
    from sayari.client import Client

    client = Client(
        client_name="YOUR_CLIENT_NAME",
        client_id="YOUR_CLIENT_ID",
        client_secret="YOUR_CLIENT_SECRET",
    )
    """
    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: SayariEnvironment = SayariEnvironment.PRODUCTION,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.Client] = None,
        client_id: str,
        client_secret: str
    ):
        super().__init__(
            base_url=base_url,
            environment=environment,
            timeout=timeout,
            follow_redirects=follow_redirects,
            httpx_client=httpx_client,
            client_id=client_id,
            client_secret=client_secret
        )

    def screen_csv(self, path_to_csv):
        # create empty result lists
        risky_entities = []
        non_risky_entities = []
        unresolved = []

        # setup multiprocessing
        set_start_method('fork')
        rows_to_process = Queue()
        results = Queue()
        num_processes = 3
        processes = []

        # Open csv
        with open(path_to_csv) as csv_file:
            csv_reader = csv.reader(csv_file)
            column_map = map_csv(next(csv_reader))
            for row in csv_reader:
                # queue up row to process
                rows_to_process.put(row)

        # start processing rows
        for i in range(num_processes):
            p = Process(target=process_row, args=(self, column_map, rows_to_process, results))
            processes.append(p)
            p.start()

        # read results
        while any(p.is_alive() for p in processes):
            try:
                result = results.get_nowait()
            except queue.Empty:
                time.sleep(1)
            else:
                # check results
                if "risky" in result:
                    risky_entities.append(result["risky"])
                elif "non_risky" in result:
                    non_risky_entities.append(result["non_risky"])
                elif "unresolved" in result:
                    unresolved.append(result["unresolved"])

        # complete processes
        for p in processes:
            p.join()

        return risky_entities, non_risky_entities, unresolved


class AsyncSayari(AsyncBaseClient):
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propogate to these functions.

    Parameters:
        - base_url: typing.Optional[str]. The base url to use for requests from the client.

        - environment: ClientEnvironment. The environment to use for requests from the client. from .environment import ClientEnvironment

                                          Defaults to ClientEnvironment.PRODUCTION

        - client_name: str.

        - timeout: typing.Optional[float]. The timeout to be used, in seconds, for requests by default the timeout is 60 seconds, unless a custom httpx client is used, in which case a default is not set.

        - follow_redirects: typing.Optional[bool]. Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

        - httpx_client: typing.Optional[httpx.AsyncClient]. The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

        - client_id: str.

        - client_secret: str.
    ---
    from sayari.client import AsyncClient

    client = AsyncClient(
        client_name="YOUR_CLIENT_NAME",
        client_id="YOUR_CLIENT_ID",
        client_secret="YOUR_CLIENT_SECRET",
    )
    """
    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: SayariEnvironment = SayariEnvironment.PRODUCTION,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.AsyncClient] = None,
        client_id: str,
        client_secret: str
    ):
        super().__init__(
            base_url=base_url,
            environment=environment,
            timeout=timeout,
            follow_redirects=follow_redirects,
            httpx_client=httpx_client,
            client_id=client_id,
            client_secret=client_secret
        )


def process_row(client, column_map, rows_to_process, results):
    while True:
        try:
            row = rows_to_process.get_nowait()
        except queue.Empty:
            break
        else:
            # attempt to resolve entity
            entity_id = resolve_entity(client, column_map, row)

            # track unresolved rows
            if entity_id is None:
                # unresolved.append(row)
                results.put({"unresolved": row})
                continue

            # Get entity summary
            entity_summary = client.entity.entity_summary(entity_id)
            if len(entity_summary.risk) > 0:
                # risky_entities.append(entity_summary)
                results.put({"risky": entity_summary})
            else:
                # non_risky_entities.append(entity_summary)
                results.put({"non_risky": entity_summary})
    return True


def map_csv(row):
    column_map = {}
    # for each column of the CSV
    i = 0
    for col in row:
        # Remove all spaces from the column name and convert to lowercase
        col_name = col.lower().replace(' ', '')
        # check if it corresponds to one of our resolution fields
        if col_name in csvDict:
            # if so, check if we have mapped that column yet
            if col_name not in column_map:
                # if not, create it in the map
                column_map[col_name] = []
            # add column index to dict
            column_map[col_name].append(i)
        else:
            raise Exception("One or more of the CSV columns does not map to a resolution field")
        i += 1
    return column_map


def resolve_entity(client, column_map, row):
    name = None
    if Name in column_map:
        name = []
        for name_col in column_map[Name]:
            name.append(row[name_col])

    identifier = None
    if Identifier in column_map:
        identifier = []
        for id_col in column_map[Identifier]:
            identifier.append(row[id_col])

    country = None
    if Country in column_map:
        country = []
        for country_col in column_map[Country]:
            country.append(row[country_col])

    address = None
    if Address in column_map:
        address = []
        for address_col in column_map[Address]:
            address.append(row[address_col])

    dob = None
    if DOB in column_map:
        dob = []
        for dob_col in column_map[DOB]:
            dob.append(row[dob_col])

    contact = None
    if Contact in column_map:
        contact = []
        for contact_col in column_map[Contact]:
            contact.append(row[contact_col])

    ent_type = None
    if Type in column_map:
        ent_type = []
        for type_col in column_map[Type]:
            ent_type.append(row[type_col])

    entity = client.resolution.resolution(name=name, identifier=identifier, country=country, address=address,
                                          date_of_birth=dob, contact=contact, type=ent_type)

    if len(entity.data) == 0:
        return None
    return entity.data[0].entity_id


def encode_record_id(record_id):
    return urllib.parse.quote(record_id, safe='')



# This file was auto-generated by Fern from our API Definition.

import typing

Country = typing.Union[
    typing.Literal[
        "ABW",
        "AFG",
        "AGO",
        "AIA",
        "ALA",
        "ALB",
        "AND",
        "ARE",
        "ARG",
        "ARM",
        "ASM",
        "ATA",
        "ATF",
        "ATG",
        "AUS",
        "AUT",
        "AZE",
        "BDI",
        "BEL",
        "BEN",
        "BFA",
        "BGD",
        "BGR",
        "BHR",
        "BHS",
        "BIH",
        "BLM",
        "SHN",
        "BLR",
        "BLZ",
        "BMU",
        "BOL",
        "BES",
        "BRA",
        "BRB",
        "BRN",
        "BTN",
        "BVT",
        "BWA",
        "CAF",
        "CAN",
        "CCK",
        "CHE",
        "CHL",
        "CHN",
        "CIV",
        "CMR",
        "COD",
        "COG",
        "COK",
        "COL",
        "COM",
        "CPV",
        "CRI",
        "CUB",
        "CUW",
        "CXR",
        "CYM",
        "CYP",
        "CZE",
        "DEU",
        "DJI",
        "DMA",
        "DNK",
        "DOM",
        "DZA",
        "ECU",
        "EGY",
        "ERI",
        "ESH",
        "ESP",
        "EST",
        "ETH",
        "FIN",
        "FJI",
        "FLK",
        "FRA",
        "FRO",
        "FSM",
        "GAB",
        "GBR",
        "GEO",
        "GGY",
        "GHA",
        "GIB",
        "GIN",
        "GLP",
        "GMB",
        "GNB",
        "GNQ",
        "GRC",
        "GRD",
        "GRL",
        "GTM",
        "GUF",
        "GUM",
        "GUY",
        "HKG",
        "HMD",
        "HND",
        "HRV",
        "HTI",
        "HUN",
        "IDN",
        "IMN",
        "IND",
        "IOT",
        "IRL",
        "IRN",
        "IRQ",
        "ISL",
        "ISR",
        "ITA",
        "JAM",
        "JEY",
        "JOR",
        "JPN",
        "KAZ",
        "KEN",
        "KGZ",
        "KHM",
        "KIR",
        "KNA",
        "KOR",
        "UNK",
        "KWT",
        "LAO",
        "LBN",
        "LBR",
        "LBY",
        "LCA",
        "LIE",
        "LKA",
        "LSO",
        "LTU",
        "LUX",
        "LVA",
        "MAC",
        "MAF",
        "MAR",
        "MCO",
        "MDA",
        "MDG",
        "MDV",
        "MEX",
        "MHL",
        "MKD",
        "MLI",
        "MLT",
        "MMR",
        "MNE",
        "MNG",
        "MNP",
        "MOZ",
        "MRT",
        "MSR",
        "MTQ",
        "MUS",
        "MWI",
        "MYS",
        "MYT",
        "NAM",
        "NCL",
        "NER",
        "NFK",
        "NGA",
        "NIC",
        "NIU",
        "NLD",
        "NOR",
        "NPL",
        "NRU",
        "NZL",
        "OMN",
        "PAK",
        "PAN",
        "PCN",
        "PER",
        "PHL",
        "PLW",
        "PNG",
        "POL",
        "PRI",
        "PRK",
        "PRT",
        "PRY",
        "PSE",
        "PYF",
        "QAT",
        "REU",
        "ROU",
        "RUS",
        "RWA",
        "SAU",
        "SDN",
        "SEN",
        "SGP",
        "SGS",
        "SJM",
        "SLB",
        "SLE",
        "SLV",
        "SMR",
        "SOM",
        "SPM",
        "SRB",
        "SSD",
        "STP",
        "SUR",
        "SVK",
        "SVN",
        "SWE",
        "SWZ",
        "SXM",
        "SYC",
        "SYR",
        "TCA",
        "TCD",
        "TGO",
        "THA",
        "TJK",
        "TKL",
        "TKM",
        "TLS",
        "TON",
        "TTO",
        "TUN",
        "TUR",
        "TUV",
        "TWN",
        "TZA",
        "UGA",
        "UKR",
        "UMI",
        "URY",
        "USA",
        "UZB",
        "VAT",
        "VCT",
        "VEN",
        "VGB",
        "VIR",
        "VNM",
        "VUT",
        "WLF",
        "WSM",
        "YEM",
        "ZAF",
        "ZMB",
        "ZWE",
        "XXX",
        "RKS",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import pydantic_v1
from ..core.query_encoder import encode_query
from ..core.remove_none_from_dict import remove_none_from_dict
from ..core.request_options import RequestOptions
from ..generated_types.types.country import Country
from ..generated_types.types.entities import Entities
from ..generated_types.types.relationships import Relationships
from ..shared_errors.errors.bad_gateway import BadGateway
from ..shared_errors.errors.bad_request import BadRequest
from ..shared_errors.errors.connection_error import ConnectionError
from ..shared_errors.errors.internal_server_error import InternalServerError
from ..shared_errors.errors.method_not_allowed import MethodNotAllowed
from ..shared_errors.errors.not_found import NotFound
from ..shared_errors.errors.rate_limit_exceeded import RateLimitExceeded
from ..shared_errors.errors.unauthorized import Unauthorized
from ..shared_errors.types.bad_gateway_response import BadGatewayResponse
from ..shared_errors.types.bad_request_response import BadRequestResponse
from ..shared_errors.types.connection_error_response import ConnectionErrorResponse
from ..shared_errors.types.internal_server_error_response import InternalServerErrorResponse
from ..shared_errors.types.method_not_allowed_response import MethodNotAllowedResponse
from ..shared_errors.types.not_found_response import NotFoundResponse
from ..shared_errors.types.rate_limit_response import RateLimitResponse
from ..shared_errors.types.unauthorized_response import UnauthorizedResponse
from .types.shortest_path_response import ShortestPathResponse
from .types.traversal_response import TraversalResponse


class TraversalClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def traversal(
        self,
        id: str,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        min_depth: typing.Optional[int] = None,
        max_depth: typing.Optional[int] = None,
        relationships: typing.Optional[typing.Union[Relationships, typing.Sequence[Relationships]]] = None,
        psa: typing.Optional[bool] = None,
        countries: typing.Optional[typing.Union[Country, typing.Sequence[Country]]] = None,
        types: typing.Optional[typing.Union[Entities, typing.Sequence[Entities]]] = None,
        sanctioned: typing.Optional[bool] = None,
        pep: typing.Optional[bool] = None,
        min_shares: typing.Optional[int] = None,
        include_unknown_shares: typing.Optional[bool] = None,
        exclude_former_relationships: typing.Optional[bool] = None,
        exclude_closed_entities: typing.Optional[bool] = None,
        eu_high_risk_third: typing.Optional[bool] = None,
        reputational_risk_modern_slavery: typing.Optional[bool] = None,
        state_owned: typing.Optional[bool] = None,
        formerly_sanctioned: typing.Optional[bool] = None,
        reputational_risk_terrorism: typing.Optional[bool] = None,
        reputational_risk_organized_crime: typing.Optional[bool] = None,
        reputational_risk_financial_crime: typing.Optional[bool] = None,
        reputational_risk_bribery_and_corruption: typing.Optional[bool] = None,
        reputational_risk_other: typing.Optional[bool] = None,
        reputational_risk_cybercrime: typing.Optional[bool] = None,
        regulatory_action: typing.Optional[bool] = None,
        law_enforcement_action: typing.Optional[bool] = None,
        xinjiang_geospatial: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TraversalResponse:
        """
        The Traversal endpoint returns paths from a single target entity to up to 50 directly or indirectly-related entities. Each path includes information on the 0 to 10 intermediary entities, as well as their connecting relationships. The response's explored_count field indicates the size of the graph subset the application searched. Running a traversal on a highly connected entity with a restrictive set of argument filters and a high max depth will require the application to explore a higher number of traversal paths, which may affect performance. In cases where a traversal searches over a very large, highly-connected subgraph, a partial result set may be returned containing only the most relevant results. This will be indicated in the response by the partial_results field.

        Parameters
        ----------
        id : str
            Unique identifier of the entity

        limit : typing.Optional[int]
            Limit total values for traversal. Defaults to 10. Max of 50.

        offset : typing.Optional[int]
            Offset values for traversal. Defaults to 0. Max of 1000.

        min_depth : typing.Optional[int]
            Set minimum depth for traversal. Defaults to 1.

        max_depth : typing.Optional[int]
            Set maximum depth for traversal. Defaults to 4.

        relationships : typing.Optional[typing.Union[Relationships, typing.Sequence[Relationships]]]
            Set relationship type(s) to follow when traversing related entities. Defaults to following all relationship types.

        psa : typing.Optional[bool]
            Also traverse relationships from entities that are possibly the same as any entity that appears in the path. Defaults to traversing possibly same as relationships.

        countries : typing.Optional[typing.Union[Country, typing.Sequence[Country]]]
            Filter paths to only those that end at an entity associated with the specified country(ies). Defaults to returning paths that end in any [country](/sayari-library/ontology/enumerated-types#country).

        types : typing.Optional[typing.Union[Entities, typing.Sequence[Entities]]]
            Filter paths to only those that end at an entity of the specified type(s). Defaults to returning paths that end at any type.

        sanctioned : typing.Optional[bool]
            Filter paths to only those that end at an entity appearing on a watchlist. Defaults to not filtering paths by sanctioned status.

        pep : typing.Optional[bool]
            Filter paths to only those that end at an entity appearing on a pep list. Defaults to not filtering paths by pep status.

        min_shares : typing.Optional[int]
            Set minimum percentage of share ownership for traversal. Defaults to 0.

        include_unknown_shares : typing.Optional[bool]
            Also traverse relationships when share percentages are unknown. Only useful when min_shares is set greater than 0. Defaults to true.

        exclude_former_relationships : typing.Optional[bool]
            Include relationships that were valid in the past but not at the present time. Defaults to true.

        exclude_closed_entities : typing.Optional[bool]
            Include entities that existed in the past but not at the present time. Defaults to false.

        eu_high_risk_third : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_modern_slavery : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        state_owned : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        formerly_sanctioned : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_terrorism : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_organized_crime : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_financial_crime : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_bribery_and_corruption : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_other : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_cybercrime : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        regulatory_action : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        law_enforcement_action : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        xinjiang_geospatial : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TraversalResponse

        Examples
        --------
        from sayari.client import Sayari

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.traversal.traversal(
            id="mGq1lpuqKssNWTjIokuPeA",
            limit=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            method="GET",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"v1/traversal/{jsonable_encoder(id)}"),
            params=encode_query(
                jsonable_encoder(
                    remove_none_from_dict(
                        {
                            "limit": limit,
                            "offset": offset,
                            "min_depth": min_depth,
                            "max_depth": max_depth,
                            "relationships": relationships,
                            "psa": psa,
                            "countries": countries,
                            "types": types,
                            "sanctioned": sanctioned,
                            "pep": pep,
                            "min_shares": min_shares,
                            "include_unknown_shares": include_unknown_shares,
                            "exclude_former_relationships": exclude_former_relationships,
                            "exclude_closed_entities": exclude_closed_entities,
                            "eu_high_risk_third": eu_high_risk_third,
                            "reputational_risk_modern_slavery": reputational_risk_modern_slavery,
                            "state_owned": state_owned,
                            "formerly_sanctioned": formerly_sanctioned,
                            "reputational_risk_terrorism": reputational_risk_terrorism,
                            "reputational_risk_organized_crime": reputational_risk_organized_crime,
                            "reputational_risk_financial_crime": reputational_risk_financial_crime,
                            "reputational_risk_bribery_and_corruption": reputational_risk_bribery_and_corruption,
                            "reputational_risk_other": reputational_risk_other,
                            "reputational_risk_cybercrime": reputational_risk_cybercrime,
                            "regulatory_action": regulatory_action,
                            "law_enforcement_action": law_enforcement_action,
                            "xinjiang_geospatial": xinjiang_geospatial,
                            **(
                                request_options.get("additional_query_parameters", {})
                                if request_options is not None
                                else {}
                            ),
                        }
                    )
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(TraversalResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFound(pydantic_v1.parse_obj_as(NotFoundResponse, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowed(pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        if _response.status_code == 502:
            raise BadGateway(pydantic_v1.parse_obj_as(BadGatewayResponse, _response.json()))  # type: ignore
        if _response.status_code == 520:
            raise ConnectionError(pydantic_v1.parse_obj_as(ConnectionErrorResponse, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def ubo(
        self,
        id: str,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        min_depth: typing.Optional[int] = None,
        max_depth: typing.Optional[int] = None,
        relationships: typing.Optional[typing.Union[Relationships, typing.Sequence[Relationships]]] = None,
        psa: typing.Optional[bool] = None,
        countries: typing.Optional[typing.Union[Country, typing.Sequence[Country]]] = None,
        types: typing.Optional[typing.Union[Entities, typing.Sequence[Entities]]] = None,
        sanctioned: typing.Optional[bool] = None,
        pep: typing.Optional[bool] = None,
        min_shares: typing.Optional[int] = None,
        include_unknown_shares: typing.Optional[bool] = None,
        exclude_former_relationships: typing.Optional[bool] = None,
        exclude_closed_entities: typing.Optional[bool] = None,
        eu_high_risk_third: typing.Optional[bool] = None,
        reputational_risk_modern_slavery: typing.Optional[bool] = None,
        state_owned: typing.Optional[bool] = None,
        formerly_sanctioned: typing.Optional[bool] = None,
        reputational_risk_terrorism: typing.Optional[bool] = None,
        reputational_risk_organized_crime: typing.Optional[bool] = None,
        reputational_risk_financial_crime: typing.Optional[bool] = None,
        reputational_risk_bribery_and_corruption: typing.Optional[bool] = None,
        reputational_risk_other: typing.Optional[bool] = None,
        reputational_risk_cybercrime: typing.Optional[bool] = None,
        regulatory_action: typing.Optional[bool] = None,
        law_enforcement_action: typing.Optional[bool] = None,
        xinjiang_geospatial: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TraversalResponse:
        """
        The UBO endpoint returns paths from a single target entity to up to 50 beneficial owners. The endpoint is a shorthand for the equivalent traversal query.

        Parameters
        ----------
        id : str
            Unique identifier of the entity

        limit : typing.Optional[int]
            Limit total values for traversal. Defaults to 10. Max of 50.

        offset : typing.Optional[int]
            Offset values for traversal. Defaults to 0. Max of 1000.

        min_depth : typing.Optional[int]
            Set minimum depth for traversal. Defaults to 1.

        max_depth : typing.Optional[int]
            Set maximum depth for traversal. Defaults to 4.

        relationships : typing.Optional[typing.Union[Relationships, typing.Sequence[Relationships]]]
            Set relationship type(s) to follow when traversing related entities. Defaults to has_shareholder, has_beneficial_owner, has_owner, subsidiary_of, and branch_of.

        psa : typing.Optional[bool]
            Also traverse relationships from entities that are possibly the same as any entity that appears in the path. Defaults to traversing possibly same as relationships.

        countries : typing.Optional[typing.Union[Country, typing.Sequence[Country]]]
            Filter paths to only those that end at an entity associated with the specified country(ies). Defaults to returning paths that end in any [country](/sayari-library/ontology/enumerated-types#country).

        types : typing.Optional[typing.Union[Entities, typing.Sequence[Entities]]]
            Filter paths to only those that end at an entity of the specified type(s). Defaults to returning paths that end at any type.

        sanctioned : typing.Optional[bool]
            Filter paths to only those that end at an entity appearing on a watchlist. Defaults to not filtering paths by sanctioned status.

        pep : typing.Optional[bool]
            Filter paths to only those that end at an entity appearing on a pep list. Defaults to not filtering paths by pep status.

        min_shares : typing.Optional[int]
            Set minimum percentage of share ownership for traversal. Defaults to 0.

        include_unknown_shares : typing.Optional[bool]
            Also traverse relationships when share percentages are unknown. Only useful when min_shares is set greater than 0. Defaults to true.

        exclude_former_relationships : typing.Optional[bool]
            Include relationships that were valid in the past but not at the present time. Defaults to true.

        exclude_closed_entities : typing.Optional[bool]
            Include entities that existed in the past but not at the present time. Defaults to false.

        eu_high_risk_third : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_modern_slavery : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        state_owned : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        formerly_sanctioned : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_terrorism : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_organized_crime : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_financial_crime : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_bribery_and_corruption : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_other : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_cybercrime : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        regulatory_action : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        law_enforcement_action : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        xinjiang_geospatial : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TraversalResponse

        Examples
        --------
        from sayari.client import Sayari

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.traversal.ubo(
            id="mGq1lpuqKssNWTjIokuPeA",
            limit=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            method="GET",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"v1/ubo/{jsonable_encoder(id)}"),
            params=encode_query(
                jsonable_encoder(
                    remove_none_from_dict(
                        {
                            "limit": limit,
                            "offset": offset,
                            "min_depth": min_depth,
                            "max_depth": max_depth,
                            "relationships": relationships,
                            "psa": psa,
                            "countries": countries,
                            "types": types,
                            "sanctioned": sanctioned,
                            "pep": pep,
                            "min_shares": min_shares,
                            "include_unknown_shares": include_unknown_shares,
                            "exclude_former_relationships": exclude_former_relationships,
                            "exclude_closed_entities": exclude_closed_entities,
                            "eu_high_risk_third": eu_high_risk_third,
                            "reputational_risk_modern_slavery": reputational_risk_modern_slavery,
                            "state_owned": state_owned,
                            "formerly_sanctioned": formerly_sanctioned,
                            "reputational_risk_terrorism": reputational_risk_terrorism,
                            "reputational_risk_organized_crime": reputational_risk_organized_crime,
                            "reputational_risk_financial_crime": reputational_risk_financial_crime,
                            "reputational_risk_bribery_and_corruption": reputational_risk_bribery_and_corruption,
                            "reputational_risk_other": reputational_risk_other,
                            "reputational_risk_cybercrime": reputational_risk_cybercrime,
                            "regulatory_action": regulatory_action,
                            "law_enforcement_action": law_enforcement_action,
                            "xinjiang_geospatial": xinjiang_geospatial,
                            **(
                                request_options.get("additional_query_parameters", {})
                                if request_options is not None
                                else {}
                            ),
                        }
                    )
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(TraversalResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFound(pydantic_v1.parse_obj_as(NotFoundResponse, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowed(pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        if _response.status_code == 502:
            raise BadGateway(pydantic_v1.parse_obj_as(BadGatewayResponse, _response.json()))  # type: ignore
        if _response.status_code == 520:
            raise ConnectionError(pydantic_v1.parse_obj_as(ConnectionErrorResponse, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def ownership(
        self,
        id: str,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        min_depth: typing.Optional[int] = None,
        max_depth: typing.Optional[int] = None,
        relationships: typing.Optional[typing.Union[Relationships, typing.Sequence[Relationships]]] = None,
        psa: typing.Optional[bool] = None,
        countries: typing.Optional[typing.Union[Country, typing.Sequence[Country]]] = None,
        types: typing.Optional[typing.Union[Entities, typing.Sequence[Entities]]] = None,
        sanctioned: typing.Optional[bool] = None,
        pep: typing.Optional[bool] = None,
        min_shares: typing.Optional[int] = None,
        include_unknown_shares: typing.Optional[bool] = None,
        exclude_former_relationships: typing.Optional[bool] = None,
        exclude_closed_entities: typing.Optional[bool] = None,
        eu_high_risk_third: typing.Optional[bool] = None,
        reputational_risk_modern_slavery: typing.Optional[bool] = None,
        state_owned: typing.Optional[bool] = None,
        formerly_sanctioned: typing.Optional[bool] = None,
        reputational_risk_terrorism: typing.Optional[bool] = None,
        reputational_risk_organized_crime: typing.Optional[bool] = None,
        reputational_risk_financial_crime: typing.Optional[bool] = None,
        reputational_risk_bribery_and_corruption: typing.Optional[bool] = None,
        reputational_risk_other: typing.Optional[bool] = None,
        reputational_risk_cybercrime: typing.Optional[bool] = None,
        regulatory_action: typing.Optional[bool] = None,
        law_enforcement_action: typing.Optional[bool] = None,
        xinjiang_geospatial: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TraversalResponse:
        """
        The Ownership endpoint returns paths from a single target entity to up to 50 entities directly or indirectly owned by that entity. The endpoint is a shorthand for the equivalent traversal query.

        Parameters
        ----------
        id : str
            Unique identifier of the entity

        limit : typing.Optional[int]
            Limit total values for traversal. Defaults to 10. Max of 50.

        offset : typing.Optional[int]
            Offset values for traversal. Defaults to 0. Max of 1000.

        min_depth : typing.Optional[int]
            Set minimum depth for traversal. Defaults to 1.

        max_depth : typing.Optional[int]
            Set maximum depth for traversal. Defaults to 4.

        relationships : typing.Optional[typing.Union[Relationships, typing.Sequence[Relationships]]]
            Set relationship type(s) to follow when traversing related entities. Defaults to shareholder_of, beneficial_owner_of, owner_of, has_subsidiary, and has_branch.

        psa : typing.Optional[bool]
            Also traverse relationships from entities that are possibly the same as any entity that appears in the path. Defaults to traversing possibly same as relationships.

        countries : typing.Optional[typing.Union[Country, typing.Sequence[Country]]]
            Filter paths to only those that end at an entity associated with the specified country(ies). Defaults to returning paths that end in any country.

        types : typing.Optional[typing.Union[Entities, typing.Sequence[Entities]]]
            Filter paths to only those that end at an entity of the specified type(s). Defaults to returning paths that end at any type.

        sanctioned : typing.Optional[bool]
            Filter paths to only those that end at an entity appearing on a watchlist. Defaults to not filtering paths by sanctioned status.

        pep : typing.Optional[bool]
            Filter paths to only those that end at an entity appearing on a pep list. Defaults to not filtering paths by pep status.

        min_shares : typing.Optional[int]
            Set minimum percentage of share ownership for traversal. Defaults to 0.

        include_unknown_shares : typing.Optional[bool]
            Also traverse relationships when share percentages are unknown. Only useful when min_shares is set greater than 0. Defaults to true.

        exclude_former_relationships : typing.Optional[bool]
            Include relationships that were valid in the past but not at the present time. Defaults to true.

        exclude_closed_entities : typing.Optional[bool]
            Include entities that existed in the past but not at the present time. Defaults to false.

        eu_high_risk_third : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_modern_slavery : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        state_owned : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        formerly_sanctioned : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_terrorism : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_organized_crime : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_financial_crime : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_bribery_and_corruption : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_other : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_cybercrime : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        regulatory_action : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        law_enforcement_action : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        xinjiang_geospatial : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TraversalResponse

        Examples
        --------
        from sayari.client import Sayari

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.traversal.ownership(
            id="mGq1lpuqKssNWTjIokuPeA",
            limit=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            method="GET",
            url=urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/downstream/{jsonable_encoder(id)}"
            ),
            params=encode_query(
                jsonable_encoder(
                    remove_none_from_dict(
                        {
                            "limit": limit,
                            "offset": offset,
                            "min_depth": min_depth,
                            "max_depth": max_depth,
                            "relationships": relationships,
                            "psa": psa,
                            "countries": countries,
                            "types": types,
                            "sanctioned": sanctioned,
                            "pep": pep,
                            "min_shares": min_shares,
                            "include_unknown_shares": include_unknown_shares,
                            "exclude_former_relationships": exclude_former_relationships,
                            "exclude_closed_entities": exclude_closed_entities,
                            "eu_high_risk_third": eu_high_risk_third,
                            "reputational_risk_modern_slavery": reputational_risk_modern_slavery,
                            "state_owned": state_owned,
                            "formerly_sanctioned": formerly_sanctioned,
                            "reputational_risk_terrorism": reputational_risk_terrorism,
                            "reputational_risk_organized_crime": reputational_risk_organized_crime,
                            "reputational_risk_financial_crime": reputational_risk_financial_crime,
                            "reputational_risk_bribery_and_corruption": reputational_risk_bribery_and_corruption,
                            "reputational_risk_other": reputational_risk_other,
                            "reputational_risk_cybercrime": reputational_risk_cybercrime,
                            "regulatory_action": regulatory_action,
                            "law_enforcement_action": law_enforcement_action,
                            "xinjiang_geospatial": xinjiang_geospatial,
                            **(
                                request_options.get("additional_query_parameters", {})
                                if request_options is not None
                                else {}
                            ),
                        }
                    )
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(TraversalResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFound(pydantic_v1.parse_obj_as(NotFoundResponse, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowed(pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        if _response.status_code == 502:
            raise BadGateway(pydantic_v1.parse_obj_as(BadGatewayResponse, _response.json()))  # type: ignore
        if _response.status_code == 520:
            raise ConnectionError(pydantic_v1.parse_obj_as(ConnectionErrorResponse, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def watchlist(
        self,
        id: str,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        min_depth: typing.Optional[int] = None,
        max_depth: typing.Optional[int] = None,
        relationships: typing.Optional[typing.Union[Relationships, typing.Sequence[Relationships]]] = None,
        psa: typing.Optional[bool] = None,
        countries: typing.Optional[typing.Union[Country, typing.Sequence[Country]]] = None,
        types: typing.Optional[typing.Union[Entities, typing.Sequence[Entities]]] = None,
        sanctioned: typing.Optional[bool] = None,
        pep: typing.Optional[bool] = None,
        min_shares: typing.Optional[int] = None,
        include_unknown_shares: typing.Optional[bool] = None,
        exclude_former_relationships: typing.Optional[bool] = None,
        exclude_closed_entities: typing.Optional[bool] = None,
        eu_high_risk_third: typing.Optional[bool] = None,
        reputational_risk_modern_slavery: typing.Optional[bool] = None,
        state_owned: typing.Optional[bool] = None,
        formerly_sanctioned: typing.Optional[bool] = None,
        reputational_risk_terrorism: typing.Optional[bool] = None,
        reputational_risk_organized_crime: typing.Optional[bool] = None,
        reputational_risk_financial_crime: typing.Optional[bool] = None,
        reputational_risk_bribery_and_corruption: typing.Optional[bool] = None,
        reputational_risk_other: typing.Optional[bool] = None,
        reputational_risk_cybercrime: typing.Optional[bool] = None,
        regulatory_action: typing.Optional[bool] = None,
        law_enforcement_action: typing.Optional[bool] = None,
        xinjiang_geospatial: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TraversalResponse:
        """
        The Watchlist endpoint returns paths from a single target entity to up to 50 other entities that appear on a watchlist. The endpoint is a shorthand for the equivalent traversal query.

        Parameters
        ----------
        id : str
            Unique identifier of the entity

        limit : typing.Optional[int]
            Limit total values for traversal. Defaults to 10. Max of 50.

        offset : typing.Optional[int]
            Offset values for traversal. Defaults to 0. Max of 1000.

        min_depth : typing.Optional[int]
            Set minimum depth for traversal. Defaults to 1.

        max_depth : typing.Optional[int]
            Set maximum depth for traversal. Defaults to 4.

        relationships : typing.Optional[typing.Union[Relationships, typing.Sequence[Relationships]]]
            Set relationship type(s) to follow when traversing related entities. Defaults to following 31 relevant relationship types covering ownership, control, and trade.

        psa : typing.Optional[bool]
            Also traverse relationships from entities that are possibly the same as any entity that appears in the path. Defaults to traversing possibly same as relationships.

        countries : typing.Optional[typing.Union[Country, typing.Sequence[Country]]]
            Filter paths to only those that end at an entity associated with the specified country(ies). Defaults to returning paths that end in any country.

        types : typing.Optional[typing.Union[Entities, typing.Sequence[Entities]]]
            Filter paths to only those that end at an entity of the specified type(s). Defaults to returning paths that end at any type.

        sanctioned : typing.Optional[bool]
            Filter paths to only those that end at an entity appearing on a watchlist. Defaults to not filtering paths by sanctioned status.

        pep : typing.Optional[bool]
            Filter paths to only those that end at an entity appearing on a pep list. Defaults to not filtering paths by pep status.

        min_shares : typing.Optional[int]
            Set minimum percentage of share ownership for traversal. Defaults to 0.

        include_unknown_shares : typing.Optional[bool]
            Also traverse relationships when share percentages are unknown. Only useful when min_shares is set greater than 0. Defaults to true.

        exclude_former_relationships : typing.Optional[bool]
            Include relationships that were valid in the past but not at the present time. Defaults to false.

        exclude_closed_entities : typing.Optional[bool]
            Include entities that existed in the past but not at the present time. Defaults to false.

        eu_high_risk_third : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_modern_slavery : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        state_owned : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        formerly_sanctioned : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_terrorism : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_organized_crime : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_financial_crime : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_bribery_and_corruption : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_other : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_cybercrime : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        regulatory_action : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        law_enforcement_action : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        xinjiang_geospatial : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TraversalResponse

        Examples
        --------
        from sayari.client import Sayari

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.traversal.watchlist(
            id="mGq1lpuqKssNWTjIokuPeA",
            limit=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            method="GET",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"v1/watchlist/{jsonable_encoder(id)}"),
            params=encode_query(
                jsonable_encoder(
                    remove_none_from_dict(
                        {
                            "limit": limit,
                            "offset": offset,
                            "min_depth": min_depth,
                            "max_depth": max_depth,
                            "relationships": relationships,
                            "psa": psa,
                            "countries": countries,
                            "types": types,
                            "sanctioned": sanctioned,
                            "pep": pep,
                            "min_shares": min_shares,
                            "include_unknown_shares": include_unknown_shares,
                            "exclude_former_relationships": exclude_former_relationships,
                            "exclude_closed_entities": exclude_closed_entities,
                            "eu_high_risk_third": eu_high_risk_third,
                            "reputational_risk_modern_slavery": reputational_risk_modern_slavery,
                            "state_owned": state_owned,
                            "formerly_sanctioned": formerly_sanctioned,
                            "reputational_risk_terrorism": reputational_risk_terrorism,
                            "reputational_risk_organized_crime": reputational_risk_organized_crime,
                            "reputational_risk_financial_crime": reputational_risk_financial_crime,
                            "reputational_risk_bribery_and_corruption": reputational_risk_bribery_and_corruption,
                            "reputational_risk_other": reputational_risk_other,
                            "reputational_risk_cybercrime": reputational_risk_cybercrime,
                            "regulatory_action": regulatory_action,
                            "law_enforcement_action": law_enforcement_action,
                            "xinjiang_geospatial": xinjiang_geospatial,
                            **(
                                request_options.get("additional_query_parameters", {})
                                if request_options is not None
                                else {}
                            ),
                        }
                    )
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(TraversalResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFound(pydantic_v1.parse_obj_as(NotFoundResponse, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowed(pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        if _response.status_code == 502:
            raise BadGateway(pydantic_v1.parse_obj_as(BadGatewayResponse, _response.json()))  # type: ignore
        if _response.status_code == 520:
            raise ConnectionError(pydantic_v1.parse_obj_as(ConnectionErrorResponse, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def shortest_path(
        self,
        *,
        entities: typing.Union[str, typing.Sequence[str]],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ShortestPathResponse:
        """
        The Shortest Path endpoint returns a response identifying the shortest traversal path connecting each pair of entities.

        Parameters
        ----------
        entities : typing.Union[str, typing.Sequence[str]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ShortestPathResponse

        Examples
        --------
        from sayari.client import Sayari

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.traversal.shortest_path(
            entities="string",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            method="GET",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/shortest_path"),
            params=encode_query(
                jsonable_encoder(
                    remove_none_from_dict(
                        {
                            "entities": entities,
                            **(
                                request_options.get("additional_query_parameters", {})
                                if request_options is not None
                                else {}
                            ),
                        }
                    )
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(ShortestPathResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFound(pydantic_v1.parse_obj_as(NotFoundResponse, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowed(pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        if _response.status_code == 502:
            raise BadGateway(pydantic_v1.parse_obj_as(BadGatewayResponse, _response.json()))  # type: ignore
        if _response.status_code == 520:
            raise ConnectionError(pydantic_v1.parse_obj_as(ConnectionErrorResponse, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncTraversalClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def traversal(
        self,
        id: str,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        min_depth: typing.Optional[int] = None,
        max_depth: typing.Optional[int] = None,
        relationships: typing.Optional[typing.Union[Relationships, typing.Sequence[Relationships]]] = None,
        psa: typing.Optional[bool] = None,
        countries: typing.Optional[typing.Union[Country, typing.Sequence[Country]]] = None,
        types: typing.Optional[typing.Union[Entities, typing.Sequence[Entities]]] = None,
        sanctioned: typing.Optional[bool] = None,
        pep: typing.Optional[bool] = None,
        min_shares: typing.Optional[int] = None,
        include_unknown_shares: typing.Optional[bool] = None,
        exclude_former_relationships: typing.Optional[bool] = None,
        exclude_closed_entities: typing.Optional[bool] = None,
        eu_high_risk_third: typing.Optional[bool] = None,
        reputational_risk_modern_slavery: typing.Optional[bool] = None,
        state_owned: typing.Optional[bool] = None,
        formerly_sanctioned: typing.Optional[bool] = None,
        reputational_risk_terrorism: typing.Optional[bool] = None,
        reputational_risk_organized_crime: typing.Optional[bool] = None,
        reputational_risk_financial_crime: typing.Optional[bool] = None,
        reputational_risk_bribery_and_corruption: typing.Optional[bool] = None,
        reputational_risk_other: typing.Optional[bool] = None,
        reputational_risk_cybercrime: typing.Optional[bool] = None,
        regulatory_action: typing.Optional[bool] = None,
        law_enforcement_action: typing.Optional[bool] = None,
        xinjiang_geospatial: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TraversalResponse:
        """
        The Traversal endpoint returns paths from a single target entity to up to 50 directly or indirectly-related entities. Each path includes information on the 0 to 10 intermediary entities, as well as their connecting relationships. The response's explored_count field indicates the size of the graph subset the application searched. Running a traversal on a highly connected entity with a restrictive set of argument filters and a high max depth will require the application to explore a higher number of traversal paths, which may affect performance. In cases where a traversal searches over a very large, highly-connected subgraph, a partial result set may be returned containing only the most relevant results. This will be indicated in the response by the partial_results field.

        Parameters
        ----------
        id : str
            Unique identifier of the entity

        limit : typing.Optional[int]
            Limit total values for traversal. Defaults to 10. Max of 50.

        offset : typing.Optional[int]
            Offset values for traversal. Defaults to 0. Max of 1000.

        min_depth : typing.Optional[int]
            Set minimum depth for traversal. Defaults to 1.

        max_depth : typing.Optional[int]
            Set maximum depth for traversal. Defaults to 4.

        relationships : typing.Optional[typing.Union[Relationships, typing.Sequence[Relationships]]]
            Set relationship type(s) to follow when traversing related entities. Defaults to following all relationship types.

        psa : typing.Optional[bool]
            Also traverse relationships from entities that are possibly the same as any entity that appears in the path. Defaults to traversing possibly same as relationships.

        countries : typing.Optional[typing.Union[Country, typing.Sequence[Country]]]
            Filter paths to only those that end at an entity associated with the specified country(ies). Defaults to returning paths that end in any [country](/sayari-library/ontology/enumerated-types#country).

        types : typing.Optional[typing.Union[Entities, typing.Sequence[Entities]]]
            Filter paths to only those that end at an entity of the specified type(s). Defaults to returning paths that end at any type.

        sanctioned : typing.Optional[bool]
            Filter paths to only those that end at an entity appearing on a watchlist. Defaults to not filtering paths by sanctioned status.

        pep : typing.Optional[bool]
            Filter paths to only those that end at an entity appearing on a pep list. Defaults to not filtering paths by pep status.

        min_shares : typing.Optional[int]
            Set minimum percentage of share ownership for traversal. Defaults to 0.

        include_unknown_shares : typing.Optional[bool]
            Also traverse relationships when share percentages are unknown. Only useful when min_shares is set greater than 0. Defaults to true.

        exclude_former_relationships : typing.Optional[bool]
            Include relationships that were valid in the past but not at the present time. Defaults to true.

        exclude_closed_entities : typing.Optional[bool]
            Include entities that existed in the past but not at the present time. Defaults to false.

        eu_high_risk_third : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_modern_slavery : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        state_owned : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        formerly_sanctioned : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_terrorism : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_organized_crime : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_financial_crime : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_bribery_and_corruption : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_other : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_cybercrime : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        regulatory_action : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        law_enforcement_action : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        xinjiang_geospatial : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TraversalResponse

        Examples
        --------
        from sayari.client import AsyncSayari

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        await client.traversal.traversal(
            id="mGq1lpuqKssNWTjIokuPeA",
            limit=1,
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            method="GET",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"v1/traversal/{jsonable_encoder(id)}"),
            params=encode_query(
                jsonable_encoder(
                    remove_none_from_dict(
                        {
                            "limit": limit,
                            "offset": offset,
                            "min_depth": min_depth,
                            "max_depth": max_depth,
                            "relationships": relationships,
                            "psa": psa,
                            "countries": countries,
                            "types": types,
                            "sanctioned": sanctioned,
                            "pep": pep,
                            "min_shares": min_shares,
                            "include_unknown_shares": include_unknown_shares,
                            "exclude_former_relationships": exclude_former_relationships,
                            "exclude_closed_entities": exclude_closed_entities,
                            "eu_high_risk_third": eu_high_risk_third,
                            "reputational_risk_modern_slavery": reputational_risk_modern_slavery,
                            "state_owned": state_owned,
                            "formerly_sanctioned": formerly_sanctioned,
                            "reputational_risk_terrorism": reputational_risk_terrorism,
                            "reputational_risk_organized_crime": reputational_risk_organized_crime,
                            "reputational_risk_financial_crime": reputational_risk_financial_crime,
                            "reputational_risk_bribery_and_corruption": reputational_risk_bribery_and_corruption,
                            "reputational_risk_other": reputational_risk_other,
                            "reputational_risk_cybercrime": reputational_risk_cybercrime,
                            "regulatory_action": regulatory_action,
                            "law_enforcement_action": law_enforcement_action,
                            "xinjiang_geospatial": xinjiang_geospatial,
                            **(
                                request_options.get("additional_query_parameters", {})
                                if request_options is not None
                                else {}
                            ),
                        }
                    )
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(TraversalResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFound(pydantic_v1.parse_obj_as(NotFoundResponse, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowed(pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        if _response.status_code == 502:
            raise BadGateway(pydantic_v1.parse_obj_as(BadGatewayResponse, _response.json()))  # type: ignore
        if _response.status_code == 520:
            raise ConnectionError(pydantic_v1.parse_obj_as(ConnectionErrorResponse, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def ubo(
        self,
        id: str,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        min_depth: typing.Optional[int] = None,
        max_depth: typing.Optional[int] = None,
        relationships: typing.Optional[typing.Union[Relationships, typing.Sequence[Relationships]]] = None,
        psa: typing.Optional[bool] = None,
        countries: typing.Optional[typing.Union[Country, typing.Sequence[Country]]] = None,
        types: typing.Optional[typing.Union[Entities, typing.Sequence[Entities]]] = None,
        sanctioned: typing.Optional[bool] = None,
        pep: typing.Optional[bool] = None,
        min_shares: typing.Optional[int] = None,
        include_unknown_shares: typing.Optional[bool] = None,
        exclude_former_relationships: typing.Optional[bool] = None,
        exclude_closed_entities: typing.Optional[bool] = None,
        eu_high_risk_third: typing.Optional[bool] = None,
        reputational_risk_modern_slavery: typing.Optional[bool] = None,
        state_owned: typing.Optional[bool] = None,
        formerly_sanctioned: typing.Optional[bool] = None,
        reputational_risk_terrorism: typing.Optional[bool] = None,
        reputational_risk_organized_crime: typing.Optional[bool] = None,
        reputational_risk_financial_crime: typing.Optional[bool] = None,
        reputational_risk_bribery_and_corruption: typing.Optional[bool] = None,
        reputational_risk_other: typing.Optional[bool] = None,
        reputational_risk_cybercrime: typing.Optional[bool] = None,
        regulatory_action: typing.Optional[bool] = None,
        law_enforcement_action: typing.Optional[bool] = None,
        xinjiang_geospatial: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TraversalResponse:
        """
        The UBO endpoint returns paths from a single target entity to up to 50 beneficial owners. The endpoint is a shorthand for the equivalent traversal query.

        Parameters
        ----------
        id : str
            Unique identifier of the entity

        limit : typing.Optional[int]
            Limit total values for traversal. Defaults to 10. Max of 50.

        offset : typing.Optional[int]
            Offset values for traversal. Defaults to 0. Max of 1000.

        min_depth : typing.Optional[int]
            Set minimum depth for traversal. Defaults to 1.

        max_depth : typing.Optional[int]
            Set maximum depth for traversal. Defaults to 4.

        relationships : typing.Optional[typing.Union[Relationships, typing.Sequence[Relationships]]]
            Set relationship type(s) to follow when traversing related entities. Defaults to has_shareholder, has_beneficial_owner, has_owner, subsidiary_of, and branch_of.

        psa : typing.Optional[bool]
            Also traverse relationships from entities that are possibly the same as any entity that appears in the path. Defaults to traversing possibly same as relationships.

        countries : typing.Optional[typing.Union[Country, typing.Sequence[Country]]]
            Filter paths to only those that end at an entity associated with the specified country(ies). Defaults to returning paths that end in any [country](/sayari-library/ontology/enumerated-types#country).

        types : typing.Optional[typing.Union[Entities, typing.Sequence[Entities]]]
            Filter paths to only those that end at an entity of the specified type(s). Defaults to returning paths that end at any type.

        sanctioned : typing.Optional[bool]
            Filter paths to only those that end at an entity appearing on a watchlist. Defaults to not filtering paths by sanctioned status.

        pep : typing.Optional[bool]
            Filter paths to only those that end at an entity appearing on a pep list. Defaults to not filtering paths by pep status.

        min_shares : typing.Optional[int]
            Set minimum percentage of share ownership for traversal. Defaults to 0.

        include_unknown_shares : typing.Optional[bool]
            Also traverse relationships when share percentages are unknown. Only useful when min_shares is set greater than 0. Defaults to true.

        exclude_former_relationships : typing.Optional[bool]
            Include relationships that were valid in the past but not at the present time. Defaults to true.

        exclude_closed_entities : typing.Optional[bool]
            Include entities that existed in the past but not at the present time. Defaults to false.

        eu_high_risk_third : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_modern_slavery : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        state_owned : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        formerly_sanctioned : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_terrorism : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_organized_crime : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_financial_crime : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_bribery_and_corruption : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_other : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_cybercrime : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        regulatory_action : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        law_enforcement_action : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        xinjiang_geospatial : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TraversalResponse

        Examples
        --------
        from sayari.client import AsyncSayari

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        await client.traversal.ubo(
            id="mGq1lpuqKssNWTjIokuPeA",
            limit=1,
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            method="GET",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"v1/ubo/{jsonable_encoder(id)}"),
            params=encode_query(
                jsonable_encoder(
                    remove_none_from_dict(
                        {
                            "limit": limit,
                            "offset": offset,
                            "min_depth": min_depth,
                            "max_depth": max_depth,
                            "relationships": relationships,
                            "psa": psa,
                            "countries": countries,
                            "types": types,
                            "sanctioned": sanctioned,
                            "pep": pep,
                            "min_shares": min_shares,
                            "include_unknown_shares": include_unknown_shares,
                            "exclude_former_relationships": exclude_former_relationships,
                            "exclude_closed_entities": exclude_closed_entities,
                            "eu_high_risk_third": eu_high_risk_third,
                            "reputational_risk_modern_slavery": reputational_risk_modern_slavery,
                            "state_owned": state_owned,
                            "formerly_sanctioned": formerly_sanctioned,
                            "reputational_risk_terrorism": reputational_risk_terrorism,
                            "reputational_risk_organized_crime": reputational_risk_organized_crime,
                            "reputational_risk_financial_crime": reputational_risk_financial_crime,
                            "reputational_risk_bribery_and_corruption": reputational_risk_bribery_and_corruption,
                            "reputational_risk_other": reputational_risk_other,
                            "reputational_risk_cybercrime": reputational_risk_cybercrime,
                            "regulatory_action": regulatory_action,
                            "law_enforcement_action": law_enforcement_action,
                            "xinjiang_geospatial": xinjiang_geospatial,
                            **(
                                request_options.get("additional_query_parameters", {})
                                if request_options is not None
                                else {}
                            ),
                        }
                    )
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(TraversalResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFound(pydantic_v1.parse_obj_as(NotFoundResponse, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowed(pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        if _response.status_code == 502:
            raise BadGateway(pydantic_v1.parse_obj_as(BadGatewayResponse, _response.json()))  # type: ignore
        if _response.status_code == 520:
            raise ConnectionError(pydantic_v1.parse_obj_as(ConnectionErrorResponse, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def ownership(
        self,
        id: str,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        min_depth: typing.Optional[int] = None,
        max_depth: typing.Optional[int] = None,
        relationships: typing.Optional[typing.Union[Relationships, typing.Sequence[Relationships]]] = None,
        psa: typing.Optional[bool] = None,
        countries: typing.Optional[typing.Union[Country, typing.Sequence[Country]]] = None,
        types: typing.Optional[typing.Union[Entities, typing.Sequence[Entities]]] = None,
        sanctioned: typing.Optional[bool] = None,
        pep: typing.Optional[bool] = None,
        min_shares: typing.Optional[int] = None,
        include_unknown_shares: typing.Optional[bool] = None,
        exclude_former_relationships: typing.Optional[bool] = None,
        exclude_closed_entities: typing.Optional[bool] = None,
        eu_high_risk_third: typing.Optional[bool] = None,
        reputational_risk_modern_slavery: typing.Optional[bool] = None,
        state_owned: typing.Optional[bool] = None,
        formerly_sanctioned: typing.Optional[bool] = None,
        reputational_risk_terrorism: typing.Optional[bool] = None,
        reputational_risk_organized_crime: typing.Optional[bool] = None,
        reputational_risk_financial_crime: typing.Optional[bool] = None,
        reputational_risk_bribery_and_corruption: typing.Optional[bool] = None,
        reputational_risk_other: typing.Optional[bool] = None,
        reputational_risk_cybercrime: typing.Optional[bool] = None,
        regulatory_action: typing.Optional[bool] = None,
        law_enforcement_action: typing.Optional[bool] = None,
        xinjiang_geospatial: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TraversalResponse:
        """
        The Ownership endpoint returns paths from a single target entity to up to 50 entities directly or indirectly owned by that entity. The endpoint is a shorthand for the equivalent traversal query.

        Parameters
        ----------
        id : str
            Unique identifier of the entity

        limit : typing.Optional[int]
            Limit total values for traversal. Defaults to 10. Max of 50.

        offset : typing.Optional[int]
            Offset values for traversal. Defaults to 0. Max of 1000.

        min_depth : typing.Optional[int]
            Set minimum depth for traversal. Defaults to 1.

        max_depth : typing.Optional[int]
            Set maximum depth for traversal. Defaults to 4.

        relationships : typing.Optional[typing.Union[Relationships, typing.Sequence[Relationships]]]
            Set relationship type(s) to follow when traversing related entities. Defaults to shareholder_of, beneficial_owner_of, owner_of, has_subsidiary, and has_branch.

        psa : typing.Optional[bool]
            Also traverse relationships from entities that are possibly the same as any entity that appears in the path. Defaults to traversing possibly same as relationships.

        countries : typing.Optional[typing.Union[Country, typing.Sequence[Country]]]
            Filter paths to only those that end at an entity associated with the specified country(ies). Defaults to returning paths that end in any country.

        types : typing.Optional[typing.Union[Entities, typing.Sequence[Entities]]]
            Filter paths to only those that end at an entity of the specified type(s). Defaults to returning paths that end at any type.

        sanctioned : typing.Optional[bool]
            Filter paths to only those that end at an entity appearing on a watchlist. Defaults to not filtering paths by sanctioned status.

        pep : typing.Optional[bool]
            Filter paths to only those that end at an entity appearing on a pep list. Defaults to not filtering paths by pep status.

        min_shares : typing.Optional[int]
            Set minimum percentage of share ownership for traversal. Defaults to 0.

        include_unknown_shares : typing.Optional[bool]
            Also traverse relationships when share percentages are unknown. Only useful when min_shares is set greater than 0. Defaults to true.

        exclude_former_relationships : typing.Optional[bool]
            Include relationships that were valid in the past but not at the present time. Defaults to true.

        exclude_closed_entities : typing.Optional[bool]
            Include entities that existed in the past but not at the present time. Defaults to false.

        eu_high_risk_third : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_modern_slavery : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        state_owned : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        formerly_sanctioned : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_terrorism : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_organized_crime : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_financial_crime : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_bribery_and_corruption : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_other : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_cybercrime : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        regulatory_action : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        law_enforcement_action : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        xinjiang_geospatial : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TraversalResponse

        Examples
        --------
        from sayari.client import AsyncSayari

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        await client.traversal.ownership(
            id="mGq1lpuqKssNWTjIokuPeA",
            limit=1,
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            method="GET",
            url=urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/downstream/{jsonable_encoder(id)}"
            ),
            params=encode_query(
                jsonable_encoder(
                    remove_none_from_dict(
                        {
                            "limit": limit,
                            "offset": offset,
                            "min_depth": min_depth,
                            "max_depth": max_depth,
                            "relationships": relationships,
                            "psa": psa,
                            "countries": countries,
                            "types": types,
                            "sanctioned": sanctioned,
                            "pep": pep,
                            "min_shares": min_shares,
                            "include_unknown_shares": include_unknown_shares,
                            "exclude_former_relationships": exclude_former_relationships,
                            "exclude_closed_entities": exclude_closed_entities,
                            "eu_high_risk_third": eu_high_risk_third,
                            "reputational_risk_modern_slavery": reputational_risk_modern_slavery,
                            "state_owned": state_owned,
                            "formerly_sanctioned": formerly_sanctioned,
                            "reputational_risk_terrorism": reputational_risk_terrorism,
                            "reputational_risk_organized_crime": reputational_risk_organized_crime,
                            "reputational_risk_financial_crime": reputational_risk_financial_crime,
                            "reputational_risk_bribery_and_corruption": reputational_risk_bribery_and_corruption,
                            "reputational_risk_other": reputational_risk_other,
                            "reputational_risk_cybercrime": reputational_risk_cybercrime,
                            "regulatory_action": regulatory_action,
                            "law_enforcement_action": law_enforcement_action,
                            "xinjiang_geospatial": xinjiang_geospatial,
                            **(
                                request_options.get("additional_query_parameters", {})
                                if request_options is not None
                                else {}
                            ),
                        }
                    )
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(TraversalResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFound(pydantic_v1.parse_obj_as(NotFoundResponse, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowed(pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        if _response.status_code == 502:
            raise BadGateway(pydantic_v1.parse_obj_as(BadGatewayResponse, _response.json()))  # type: ignore
        if _response.status_code == 520:
            raise ConnectionError(pydantic_v1.parse_obj_as(ConnectionErrorResponse, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def watchlist(
        self,
        id: str,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        min_depth: typing.Optional[int] = None,
        max_depth: typing.Optional[int] = None,
        relationships: typing.Optional[typing.Union[Relationships, typing.Sequence[Relationships]]] = None,
        psa: typing.Optional[bool] = None,
        countries: typing.Optional[typing.Union[Country, typing.Sequence[Country]]] = None,
        types: typing.Optional[typing.Union[Entities, typing.Sequence[Entities]]] = None,
        sanctioned: typing.Optional[bool] = None,
        pep: typing.Optional[bool] = None,
        min_shares: typing.Optional[int] = None,
        include_unknown_shares: typing.Optional[bool] = None,
        exclude_former_relationships: typing.Optional[bool] = None,
        exclude_closed_entities: typing.Optional[bool] = None,
        eu_high_risk_third: typing.Optional[bool] = None,
        reputational_risk_modern_slavery: typing.Optional[bool] = None,
        state_owned: typing.Optional[bool] = None,
        formerly_sanctioned: typing.Optional[bool] = None,
        reputational_risk_terrorism: typing.Optional[bool] = None,
        reputational_risk_organized_crime: typing.Optional[bool] = None,
        reputational_risk_financial_crime: typing.Optional[bool] = None,
        reputational_risk_bribery_and_corruption: typing.Optional[bool] = None,
        reputational_risk_other: typing.Optional[bool] = None,
        reputational_risk_cybercrime: typing.Optional[bool] = None,
        regulatory_action: typing.Optional[bool] = None,
        law_enforcement_action: typing.Optional[bool] = None,
        xinjiang_geospatial: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TraversalResponse:
        """
        The Watchlist endpoint returns paths from a single target entity to up to 50 other entities that appear on a watchlist. The endpoint is a shorthand for the equivalent traversal query.

        Parameters
        ----------
        id : str
            Unique identifier of the entity

        limit : typing.Optional[int]
            Limit total values for traversal. Defaults to 10. Max of 50.

        offset : typing.Optional[int]
            Offset values for traversal. Defaults to 0. Max of 1000.

        min_depth : typing.Optional[int]
            Set minimum depth for traversal. Defaults to 1.

        max_depth : typing.Optional[int]
            Set maximum depth for traversal. Defaults to 4.

        relationships : typing.Optional[typing.Union[Relationships, typing.Sequence[Relationships]]]
            Set relationship type(s) to follow when traversing related entities. Defaults to following 31 relevant relationship types covering ownership, control, and trade.

        psa : typing.Optional[bool]
            Also traverse relationships from entities that are possibly the same as any entity that appears in the path. Defaults to traversing possibly same as relationships.

        countries : typing.Optional[typing.Union[Country, typing.Sequence[Country]]]
            Filter paths to only those that end at an entity associated with the specified country(ies). Defaults to returning paths that end in any country.

        types : typing.Optional[typing.Union[Entities, typing.Sequence[Entities]]]
            Filter paths to only those that end at an entity of the specified type(s). Defaults to returning paths that end at any type.

        sanctioned : typing.Optional[bool]
            Filter paths to only those that end at an entity appearing on a watchlist. Defaults to not filtering paths by sanctioned status.

        pep : typing.Optional[bool]
            Filter paths to only those that end at an entity appearing on a pep list. Defaults to not filtering paths by pep status.

        min_shares : typing.Optional[int]
            Set minimum percentage of share ownership for traversal. Defaults to 0.

        include_unknown_shares : typing.Optional[bool]
            Also traverse relationships when share percentages are unknown. Only useful when min_shares is set greater than 0. Defaults to true.

        exclude_former_relationships : typing.Optional[bool]
            Include relationships that were valid in the past but not at the present time. Defaults to false.

        exclude_closed_entities : typing.Optional[bool]
            Include entities that existed in the past but not at the present time. Defaults to false.

        eu_high_risk_third : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_modern_slavery : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        state_owned : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        formerly_sanctioned : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_terrorism : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_organized_crime : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_financial_crime : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_bribery_and_corruption : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_other : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        reputational_risk_cybercrime : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        regulatory_action : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        law_enforcement_action : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        xinjiang_geospatial : typing.Optional[bool]
            Filter paths to only those that entity with an entity that we have flagged with this risk factor

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TraversalResponse

        Examples
        --------
        from sayari.client import AsyncSayari

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        await client.traversal.watchlist(
            id="mGq1lpuqKssNWTjIokuPeA",
            limit=1,
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            method="GET",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"v1/watchlist/{jsonable_encoder(id)}"),
            params=encode_query(
                jsonable_encoder(
                    remove_none_from_dict(
                        {
                            "limit": limit,
                            "offset": offset,
                            "min_depth": min_depth,
                            "max_depth": max_depth,
                            "relationships": relationships,
                            "psa": psa,
                            "countries": countries,
                            "types": types,
                            "sanctioned": sanctioned,
                            "pep": pep,
                            "min_shares": min_shares,
                            "include_unknown_shares": include_unknown_shares,
                            "exclude_former_relationships": exclude_former_relationships,
                            "exclude_closed_entities": exclude_closed_entities,
                            "eu_high_risk_third": eu_high_risk_third,
                            "reputational_risk_modern_slavery": reputational_risk_modern_slavery,
                            "state_owned": state_owned,
                            "formerly_sanctioned": formerly_sanctioned,
                            "reputational_risk_terrorism": reputational_risk_terrorism,
                            "reputational_risk_organized_crime": reputational_risk_organized_crime,
                            "reputational_risk_financial_crime": reputational_risk_financial_crime,
                            "reputational_risk_bribery_and_corruption": reputational_risk_bribery_and_corruption,
                            "reputational_risk_other": reputational_risk_other,
                            "reputational_risk_cybercrime": reputational_risk_cybercrime,
                            "regulatory_action": regulatory_action,
                            "law_enforcement_action": law_enforcement_action,
                            "xinjiang_geospatial": xinjiang_geospatial,
                            **(
                                request_options.get("additional_query_parameters", {})
                                if request_options is not None
                                else {}
                            ),
                        }
                    )
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(TraversalResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFound(pydantic_v1.parse_obj_as(NotFoundResponse, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowed(pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        if _response.status_code == 502:
            raise BadGateway(pydantic_v1.parse_obj_as(BadGatewayResponse, _response.json()))  # type: ignore
        if _response.status_code == 520:
            raise ConnectionError(pydantic_v1.parse_obj_as(ConnectionErrorResponse, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def shortest_path(
        self,
        *,
        entities: typing.Union[str, typing.Sequence[str]],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ShortestPathResponse:
        """
        The Shortest Path endpoint returns a response identifying the shortest traversal path connecting each pair of entities.

        Parameters
        ----------
        entities : typing.Union[str, typing.Sequence[str]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ShortestPathResponse

        Examples
        --------
        from sayari.client import AsyncSayari

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        await client.traversal.shortest_path(
            entities="string",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            method="GET",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/shortest_path"),
            params=encode_query(
                jsonable_encoder(
                    remove_none_from_dict(
                        {
                            "entities": entities,
                            **(
                                request_options.get("additional_query_parameters", {})
                                if request_options is not None
                                else {}
                            ),
                        }
                    )
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(ShortestPathResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFound(pydantic_v1.parse_obj_as(NotFoundResponse, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowed(pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        if _response.status_code == 502:
            raise BadGateway(pydantic_v1.parse_obj_as(BadGatewayResponse, _response.json()))  # type: ignore
        if _response.status_code == 520:
            raise ConnectionError(pydantic_v1.parse_obj_as(ConnectionErrorResponse, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from .delete_resource_response import DeleteResourceResponse
from .entity_response_data import EntityResponseData
from .resource_type import ResourceType
from .save_entity_request import SaveEntityRequest
from .save_entity_response import SaveEntityResponse

__all__ = ["DeleteResourceResponse", "EntityResponseData", "ResourceType", "SaveEntityRequest", "SaveEntityResponse"]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from ...core.pydantic_utilities import deep_union_pydantic_dicts, pydantic_v1
from .project_notification_data import ProjectNotificationData


class ProjectNotificationsResponse(pydantic_v1.BaseModel):
    """
    OK

    Examples
    --------
    from sayari import (
        Notification,
        ProjectNotificationData,
        ProjectNotificationsResponse,
    )

    ProjectNotificationsResponse(
        offset=0,
        limit=20,
        next=False,
        data=[
            ProjectNotificationData(
                id="dlOL1cZzEkIEZcRUcrBZCQ",
                entity_id="dlOL1cZzEkIEZcRUcrBZCQ",
                resource_id="0eZQ43",
                notifications=[
                    Notification(
                        type="risk",
                        field="forced_labor_xinjiang_origin_subtier",
                        values=["false"],
                        date="2024-02-06T00:00:00.000Z",
                    )
                ],
            ),
            ProjectNotificationData(
                id="wxwqZshCF4trlrmOa2eu9w",
                entity_id="wxwqZshCF4trlrmOa2eu9w",
                resource_id="03ePyj",
                notifications=[
                    Notification(
                        type="risk",
                        field="forced_labor_sheffield_hallam_university_reports_origin_subtier",
                        values=["3"],
                        date="2024-02-15T00:00:00.000Z",
                    )
                ],
            ),
            ProjectNotificationData(
                id="dX9cfM3FPefIp8VAuBKgIQ",
                entity_id="dX9cfM3FPefIp8VAuBKgIQ",
                resource_id="0XEQaX",
                notifications=[
                    Notification(
                        type="risk",
                        field="forced_labor_sheffield_hallam_university_reports_origin_direct",
                        values=["1"],
                        date="2024-02-15T00:00:00.000Z",
                    ),
                    Notification(
                        type="risk",
                        field="forced_labor_sheffield_hallam_university_reports_origin_subtier",
                        values=["false"],
                        date="2024-02-15T00:00:00.000Z",
                    ),
                    Notification(
                        type="risk",
                        field="owner_of_regulatory_action_entity",
                        values=["1"],
                        date="2024-02-06T00:00:00.000Z",
                    ),
                ],
            ),
        ],
    )
    """

    offset: int
    limit: int
    next: bool
    data: typing.List[ProjectNotificationData]

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...base_types.types.paginated_response import PaginatedResponse
from ...core.datetime_utils import serialize_datetime
from ...core.pydantic_utilities import deep_union_pydantic_dicts, pydantic_v1
from .source import Source


class ListSourcesResponse(PaginatedResponse):
    """
    OK

    Examples
    --------
    from sayari import ListSourcesResponse, QualifiedCount, Source

    ListSourcesResponse(
        offset=0,
        limit=2,
        size=QualifiedCount(
            count=547,
            qualifier="eq",
        ),
        next=True,
        data=[
            Source(
                id="e85d865943ee6d8369307569d2ad9de0",
                label="Acuris Risk Intelligence Adverse Media Data",
                description="Contains PDFs and URLs to adverse media reporting for PEPs, SOEs, sanctioned entities, and entities linked to financial regulatory and law enforcement actions. Available for millions of entities from 'Acuris Risk Intelligence KYC6 (3rd Party Data)' in 'Records' section.",
                country="XXX",
                region="international_(multi-region_coverage)",
                date_added="2022-04-11",
                source_type="adverse_media_/_negative_news_data",
                record_type="adverse_media_record",
                structure="unstructured",
                source_url="https://www.acurisriskintelligence.com/",
                pep=False,
                watchlist=False,
            ),
            Source(
                id="a8c6ee1cd4dfc952105ee8c0e4836f08",
                label="Acuris Risk Intelligence KYC6 (3rd Party Data)",
                description="Contains profiles of PEPs, sanctioned entities, SOEs, and entities linked to financial regulatory and law enforcement actions from hundreds of international watchlists. Provides identifying information on individuals and companies as available.",
                country="XXX",
                region="international_(multi-region_coverage)",
                date_added="2022-02-09",
                source_type="risk_intelligence_data",
                record_type="risk_intelligence_record",
                structure="unstructured",
                source_url="https://www.acurisriskintelligence.com/",
                pep=False,
                watchlist=False,
            ),
        ],
    )
    """

    offset: int
    next: bool
    data: typing.List[Source]

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}
